# Simple demo for interfacing a 4x4 matrix keypad with
# MCP23008 or MCP23017 GPIO Expander
#
# The concept and the technical details are based on the
# Application Note AN1081a of Microchip Technology Inc.
# Reference: https://www.microchip.com/en-us/application-notes/an1081
#
# The MCP23017 is driven by the Adafruit_CircuitPython_MCP230xx library
# Reference: https://github.com/adafruit/Adafruit_CircuitPython_MCP230xx
#
# Tested with: STM32F411CE Black Pill with Flash and CircuitPython v7.3.3

import time
import board
import busio
from digitalio import *
from adafruit_mcp230xx.mcp23017 import MCP23017

decoder = {}
decoder[0x77] = 1
decoder[0x7B] = 2
decoder[0x7D] = 3
decoder[0x7E] = 0XA
decoder[0xB7] = 4
decoder[0xBB] = 5
decoder[0xBD] = 6
decoder[0xBE] = 0XB
decoder[0xD7] = 7
decoder[0xDB] = 8
decoder[0xDD] = 9
decoder[0xDE] = 0XC
decoder[0xE7] = 0XE
decoder[0xEB] = 0
decoder[0xED] = 0XF
decoder[0xEE] = 0XD


# board.A1 accepts the interrupt signal of the MCP230xx
irq_a = DigitalInOut(board.A1)
irq_a.direction = Direction.INPUT
irq_a.pull = Pull.UP

# Initialize the I2C bus and the MCP23017 device:
i2c = busio.I2C(board.SCL, board.SDA)
mcp = MCP23017(i2c, address=0x20, reset=True)    # MCP23017

def gpio_init():
    mcp.iocon = 0
    mcp.iodira = 0xF0
    mcp.gppua = 0xF0
    mcp.interrupt_configuration = 0xF0
    mcp.default_value = 0xF0
    mcp.interrupt_enable = 0xF0

def key_scan()-> int:
    r = (mcp.gpioa) & 0xF0 # Read row
    #--- Flip I/O ---------------------
    mcp.iodira = 0x0F
    mcp.gppua = 0x0F
    mcp.interrupt_configuration = 0x0F
    mcp.default_value = 0x0F
    mcp.interrupt_enable = 0x0F
    #--- Determine key code
    c = (mcp.gpioa) & 0x0F # Read column
    print("Row: {0}, Column: {1}".format(hex(r>>4),hex(c)))
    #--- Re-initialize I/O ------------
    mcp.iodira = 0xF0
    mcp.gppua = 0xF0
    mcp.interrupt_configuration = 0xF0
    mcp.default_value = 0xF0
    mcp.interrupt_enable = 0xF0
    return (r | c)

def wait_for_release():
    while not irq_a.value:
        time.sleep(0.005)
        dummy = mcp.int_capa

gpio_init()

while True:
    if not irq_a.value:
        # print("IRQ A went off")
        keycode = key_scan()
        if keycode in decoder:
            ch = decoder[keycode]
            print("code = {0}".format(hex(ch)))
        wait_for_release()