# Simple calculator by using a 4x4 matrix keypad 
# interfaced by an MCP23017 GPIO Expander.
# The ouput is simply written to the terminal 
# by using the print command.
#
# The concept and the technical details of handling
# the 4x4 key matrix is based on the
# Application Note AN1081a of Microchip Technology Inc.
# Reference: https://www.microchip.com/en-us/application-notes/an1081
#
# The calculator is a very simple one based on the article:
# Basic calculator program using Python
# Reference: https://www.geeksforgeeks.org/make-simple-calculator-using-python/
#
# The MCP23017 is driven by the Adafruit_CircuitPython_MCP230xx library
# Reference: https://github.com/adafruit/Adafruit_CircuitPython_MCP230xx
#
# Tested with: STM32F411CE Black Pill with Flash and CircuitPython v7.3.3

import time
import board
import busio
from digitalio import *
from adafruit_mcp230xx.mcp23017 import MCP23017

decoder = {}
decoder[0x77] = '1'
decoder[0x7B] = '2'
decoder[0x7D] = '3'
decoder[0x7E] = '+'
decoder[0xB7] = '4'
decoder[0xBB] = '5'
decoder[0xBD] = '6'
decoder[0xBE] = '-'
decoder[0xD7] = '7'
decoder[0xDB] = '8'
decoder[0xDD] = '9'
decoder[0xDE] = '*'
decoder[0xE7] = 'C'
decoder[0xEB] = '0'
decoder[0xED] = '='
decoder[0xEE] = '/'

entry = "0"

# board.A1 accepts the interrupt signal of the MCP230xx
irq_a = DigitalInOut(board.A1)
irq_a.direction = Direction.INPUT
irq_a.pull = Pull.UP

# Initialize the I2C bus:
i2c = busio.I2C(board.SCL, board.SDA)
mcp = MCP23017(i2c, address=0x20, reset=True)    # MCP23017


def gpio_init():
    mcp.iocon = 0
    mcp.iodira = 0xF0
    mcp.gppua = 0xF0
    mcp.gpioa = 0
    mcp.interrupt_configuration = 0xF0
    mcp.default_value = 0xF0
    mcp.interrupt_enable = 0xF0

def key_scan()-> int:
    r = (mcp.gpioa) & 0xF0 # Read row
    #--- Flip I/O ---------------------
    mcp.iodira = 0x0F
    mcp.gppua = 0x0F
    mcp.interrupt_configuration = 0x0F
    mcp.default_value = 0x0F
    mcp.interrupt_enable = 0x0F
    #--- Determine key code
    c = (mcp.gpioa) & 0x0F # Read column
    # print("Row: {0}, Column: {1}".format(hex(r>>4),hex(c)))
    #--- Re-initialize I/O ------------
    mcp.iodira = 0xF0
    mcp.gppua = 0xF0
    mcp.interrupt_configuration = 0xF0
    mcp.default_value = 0xF0
    mcp.interrupt_enable = 0xF0
    return (r | c)

def wait_for_release():
    while not irq_a.value:
        time.sleep(0.005)
        dummy = mcp.int_capa

gpio_init()
print(entry)

while True:
    if not irq_a.value:
        keycode = key_scan()
        if keycode in decoder:
            ch = decoder[keycode]
            if ch == 'C':
                entry = '0'
            elif ch == '=':
                try:
                    y = str(eval(entry))
                    entry = y
                except SyntaxError:
                    print('Syntax error!')
                    entry ='0'
                except ZeroDivisionError:
                    print('division by zero')
                    entry = '0'
            else:
                if entry == '0' and ch in '0123456789':
                    entry = ''
                entry = entry + ch
        wait_for_release()
        print(entry)