# Simple demo for interfacing a 16x2 LCD with
# MCP23008 or MCP23017 GPIO Expander
#
# Usage of the MCP2300xx as if they were native CircuitPython 
# digital inputs/outputs is explained in the tutorial of Tony DiCola.
# Reference: https://learn.adafruit.com/using-mcp23008-mcp23017-with-circuitpython/python-circuitpython
# Usage of the alphanumeric LCDs is explained in the tutorial of Lady Ada
# Reference: https://learn.adafruit.com/character-lcds/python-circuitpython
#
# The LCD is driven by the Adafruit_CircuitPython_CharLCD library
# Reference: https://github.com/adafruit/Adafruit_CircuitPython_CharLCD
# This demo is basen on the example code: lcd_mono_simpletest.py
# The MCP23017 is driven by the Adafruit_CircuitPython_MCP230xx library
# Reference: https://github.com/adafruit/Adafruit_CircuitPython_MCP230xx
#
# Tested with: STM32F411CE Black Pill with Flash and CircuitPython v7.3.3

import time
import board
import busio
from digitalio import *
import adafruit_character_lcd.character_lcd as characterlcd
from adafruit_mcp230xx.mcp23017 import MCP23017

# Initialize the I2C bus:
i2c = busio.I2C(board.SCL, board.SDA)
mcp = MCP23017(i2c, address=0x20, reset=True)    # MCP23017

# Modify this if you have a different sized character LCD
lcd_columns = 16
lcd_rows = 2

# LCD is connected to the GPIOB port of the MCP23017 IC:
lcd_rs = mcp.get_pin(8)    # B0
lcd_en = mcp.get_pin(9)    # B1
lcd_d7 = mcp.get_pin(15)   # B7
lcd_d6 = mcp.get_pin(14)   # B6
lcd_d5 = mcp.get_pin(13)   # B5
lcd_d4 = mcp.get_pin(12)   # B4
lcd_backlight = mcp.get_pin(10)   # B2

# Initialise the LCD class
lcd = characterlcd.Character_LCD_Mono(
    lcd_rs, lcd_en, lcd_d4, lcd_d5, lcd_d6, lcd_d7, lcd_columns, lcd_rows, lcd_backlight
)

# Turn backlight on
lcd.backlight = True
# Print a two line message
lcd.message = "Hello\nCircuitPython"
# Wait 5s
time.sleep(5)
lcd.clear()
# Print two line message right to left
lcd.text_direction = lcd.RIGHT_TO_LEFT
lcd.message = "Hello\nCircuitPython"
# Wait 5s
time.sleep(5)
# Return text direction to left to right
lcd.text_direction = lcd.LEFT_TO_RIGHT
# Display cursor
lcd.clear()
lcd.cursor = True
lcd.message = "Cursor! "
# Wait 5s
time.sleep(5)
# Display blinking cursor
lcd.clear()
lcd.blink = True
lcd.message = "Blinky Cursor!"
# Wait 5s
time.sleep(5)
lcd.blink = False
lcd.clear()
# Create message to scroll
scroll_msg = "<-- Scroll"
lcd.message = scroll_msg
# Scroll message to the left
for i in range(len(scroll_msg)):
    time.sleep(0.5)
    lcd.move_left()
lcd.clear()
lcd.message = "Going to sleep\nCya later!"
time.sleep(3)
# Turn backlight off
lcd.backlight = False
time.sleep(2)